"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseModel = void 0;
const lodash_1 = __importDefault(require("lodash"));
const nanoid_1 = require("nanoid");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
var RecordStatus;
(function (RecordStatus) {
    RecordStatus["ACTIVE"] = "ACTIVE";
    RecordStatus["INACTIVE"] = "INACTIVE";
})(RecordStatus || (RecordStatus = {}));
class BaseModel {
    constructor(prisma, formatErrorHandler, modelName, formatLogHandler) {
        this.prisma = prisma;
        this.modelName = modelName;
        this.formatErrorHandler = formatErrorHandler;
        this.formatLogHandler = formatLogHandler;
    }
    async add(input, requestedBy) {
        try {
            const now = new Date(new Date().toISOString());
            const data = {
                uuid: (0, nanoid_1.nanoid)(),
                createdBy: requestedBy,
                createdAt: now,
                updatedBy: requestedBy,
                updatedAt: now,
                ...input
            };
            const dbData = await this.prisma.create({ data: data });
            if (this.formatLogHandler) {
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => this.formatLogHandler(this.modelName, 'add', [input, requestedBy], dbData));
            }
            return dbData;
        }
        catch (error) {
            this.formatErrorHandler(error, this.modelName, 'add', [input, requestedBy]);
        }
    }
    async read(where) {
        try {
            const dbData = await this.prisma.findUnique({ where });
            if (this.formatLogHandler) {
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => this.formatLogHandler(this.modelName, 'read', [where], dbData));
            }
            return dbData;
        }
        catch (error) {
            this.formatErrorHandler(error, this.modelName, 'read', [where]);
        }
    }
    async browse(pagination, where, options) {
        options = lodash_1.default.cloneDeep(options);
        try {
            pagination ??= {};
            pagination.page ??= 1;
            pagination.pageSize ??= 10;
            where = {
                status: RecordStatus.ACTIVE,
                ...where
            };
            const skip = (pagination.page - 1) * pagination.pageSize;
            options.scanAll ??= false;
            options.scanAll = pagination.pageSize === 0 ? true : false;
            options.sort ??= { createdAt: 'asc' };
            options.speedMode ??= false;
            let count = 0;
            if (!options.speedMode) {
                count = await this.prisma.count({ where });
            }
            let configuration = { orderBy: [options.sort], where };
            if (!options.scanAll) {
                configuration = { ...configuration, take: pagination.pageSize, skip };
            }
            const dbData = {
                items: await this.prisma.findMany(configuration),
                properties: options.speedMode ? { ...pagination } : { ...pagination, count }
            };
            if (this.formatLogHandler) {
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => this.formatLogHandler(this.modelName, 'browse', [pagination, where, options], dbData.items));
            }
            return dbData;
        }
        catch (error) {
            this.formatErrorHandler(error, this.modelName, 'browse', [pagination, where, options]);
        }
    }
    async edit(input, where, requestedBy) {
        try {
            delete input['id'];
            delete input['uuid'];
            const data = { ...input, updatedBy: requestedBy };
            const dbData = await this.prisma.update({ where, data });
            if (this.formatLogHandler) {
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => this.formatLogHandler(this.modelName, 'edit', [input, where, requestedBy], dbData));
            }
            return dbData;
        }
        catch (error) {
            this.formatErrorHandler(error, this.modelName, 'edit', [input, where, requestedBy]);
        }
    }
    async delete(where) {
        try {
            const dbData = await this.prisma.delete({ where });
            if (this.formatLogHandler) {
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => this.formatLogHandler(this.modelName, 'delete', [where], dbData));
            }
            return dbData;
        }
        catch (error) {
            this.formatErrorHandler(error, this.modelName, 'delete', [where]);
        }
    }
    async activate(where, requestedBy) {
        try {
            const data = {
                status: RecordStatus.ACTIVE,
                updatedBy: requestedBy
            };
            const dbData = await this.edit(data, where, requestedBy);
            if (this.formatLogHandler) {
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => this.formatLogHandler(this.modelName, 'activate', [where, requestedBy], dbData));
            }
            return dbData;
        }
        catch (error) {
            this.formatErrorHandler(error, this.modelName, 'activate', [where, requestedBy]);
        }
    }
    async deactivate(where, requestedBy) {
        try {
            const data = {
                status: RecordStatus.INACTIVE,
                updatedBy: requestedBy
            };
            const dbData = await this.edit(data, where, requestedBy);
            if (this.formatLogHandler) {
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => this.formatLogHandler(this.modelName, 'deactivate', [where, requestedBy], dbData));
            }
            return dbData;
        }
        catch (error) {
            this.formatErrorHandler(error, this.modelName, 'deactivate', [where, requestedBy]);
        }
    }
}
exports.BaseModel = BaseModel;
//# sourceMappingURL=model.js.map